#!/usr/bin/env python

import h5py
import numpy as np
import importlib

import torch

torch.cuda.empty_cache()

from torch.utils.data import Dataset
from transformers import AutoTokenizer
from transformers import TrainingArguments, Trainer
from transformers import DefaultDataCollator

from modeling_bert import BertForSequenceClassification

#from LoopPredictionDataset import LoopPredictionDataset


from transformers.integrations import TensorBoardCallback

tokenizer = AutoTokenizer.from_pretrained("AIRI-Institute/gena-lm-bert-base-t2t")

model = BertForSequenceClassification.from_pretrained("AIRI-Institute/gena-lm-bert-base-t2t", 
                                                   num_labels=1,
                                                   problem_type = 'regression')

train_d = LoopPredictionDataset("/data/aapopov/Projects/GENA_loop_prediction/Homo_sapiens_pairs_clear_train.hdf5")
valid_d = LoopPredictionDataset_validation("/data/aapopov/Projects/GENA_loop_prediction/Homo_sapiens_pairs_clear_valid.hdf5")

data_collator = DefaultDataCollator()



training_args = TrainingArguments(
    output_dir="/data/aapopov/Projects/GENA_loop_prediction/fine_tuning/",
    learning_rate=2e-05,
    lr_scheduler_type="constant_with_warmup",
    warmup_ratio=0.1,
    optim="adamw_torch",
    weight_decay=0.0,
    per_device_train_batch_size=16,
    per_device_eval_batch_size=16,
    num_train_epochs=5,
    evaluation_strategy="steps",
    eval_steps=500,
    save_strategy="steps",
    save_steps=500,
    logging_strategy="steps",
    logging_steps=500,
    load_best_model_at_end=True,
    label_names=["labels"],
    dataloader_num_workers=16,
)

trainer = Trainer(
    model=model,
    args=training_args,
    train_dataset=train_d,
    eval_dataset=valid_d,
    tokenizer=tokenizer,
    data_collator=data_collator,
    callbacks=[TensorBoardCallback],
)

trainer.train()

train_d.close()
valid_d.close()